

package gov.nasa.worldwind.render;

import gov.nasa.worldwind.geom.*;
import gov.nasa.worldwind.globes.Globe;
import gov.nasa.worldwind.util.*;

import java.lang.*;


public class SurfaceSensor extends SurfaceEllipse
{

	private double outerRadius, holeRadius;
    protected Angle FOV;


	public SurfaceSensor(ShapeAttributes attributes, LatLon position, double holeRadius, double outerRadius, Angle heading, int intervals, Angle FOV)
	{
		super(attributes, position, outerRadius, outerRadius, heading, intervals);

        if (holeRadius < 0)
		{
			String message = Logging.getMessage("generic.ArgumentOutOfRange", "holeRadius < 0");
		    Logging.logger().severe(message);
		    throw new IllegalArgumentException(message);
        }

        if (FOV == null)
		{
			String message = Logging.getMessage("nullValue.FOVIsNull");
		     Logging.logger().severe(message);
		     throw new IllegalArgumentException(message);
        }

		this.holeRadius = holeRadius;
        this.FOV = FOV;
	}

		/*//Contructor to handle all previous contructor calls
		public SurfaceEllipse(ShapeAttributes attributes, LatLon center, double majorRadius, double minorRadius,
					Angle heading, int intervals)
		{
			this(attributes, center, majorRadius, minorRadius, heading, intervals, 0, Angle.ZERO);
		}*/

	public SurfaceSensor(LatLon position, double holeRadius, double outerRadius, Angle heading, int intervals, Angle FOV)
	{
		this(new BasicShapeAttributes(), position, holeRadius, outerRadius, heading, intervals, FOV);

	}

	public void setCircleAngle(Angle fov)
    {
        if (fov == null)
        {
            String message = Logging.getMessage("nullValue.HeadingIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        this.FOV = FOV;
        this.onShapeChanged();
    }

    public void setHoleRadius(double holeRadius)
	{
		if (holeRadius < 0)
		{
			String message = Logging.getMessage("generic.ArgumentOutOfRange", "holeRadius< 0");
			Logging.logger().severe(message);
		    throw new IllegalArgumentException(message);
		}
			this.holeRadius = holeRadius;
	}


	//Finds lat long of the points along edge of sensor holes
	protected LatLon computeLocationForHole(Globe globe, Angle angle)
	{
		if (globe == null)
		{
			String message = Logging.getMessage("nullValue.GlobeIsNull");
			Logging.logger().severe(message);
			throw new IllegalArgumentException(message);
		}

		if (angle == null)
		{
			String message = Logging.getMessage("nullValue.AngleIsNull");
			Logging.logger().severe(message);
			throw new IllegalArgumentException(message);
		}


		double xLength = this.holeRadius * Math.cos(angle.radians);
		double yLength = this.holeRadius * Math.sin(angle.radians);
		double distance = Math.sqrt(xLength * xLength + yLength * yLength);
		double globeRadius = globe.getRadiusAt(this.center.getLatitude(), this.center.getLongitude());

		// azimuth runs positive clockwise from north and through 360 degrees.
		double azimuth = (Math.PI / 2.0) - (Math.acos(xLength / distance) * Math.signum(yLength)
			- this.heading.radians);

		return LatLon.greatCircleEndPosition(this.center, azimuth, distance / globeRadius);
	}

	//Creates a arary that contains the coordinates which "draw" the sensor footprint
    protected Iterable<LatLon> computeLocationsHole(Globe globe, int intervals)
    {
		//y is half the outside circle locations, z is sensor hole locations
		int z,y;

		if (globe == null)
		{
			String message = Logging.getMessage("nullValue.GlobeIsNull");
			Logging.logger().severe(message);
			throw new IllegalArgumentException(message);
		}

		int numPositions = 1 + Math.max(ELLIPSE_MIN_NUM_INTERVALS, intervals);
		LatLon[] locations = new LatLon[numPositions];

		y=(int)Math.floor((numPositions-5)/3);
		z=(int)(numPositions)-2*y;

		//Outside left side circle
		double da = (this.FOV.radians) / 2 / (y-1);

		for (int i = 0;i < y;i++)
		{
			double angle = i*da;
			locations[i] = this.computeLocationFor(globe, Angle.fromRadians(angle));
		}

		//Sensor hole

		da = this.FOV.radians/(z-1);
		double half = this.FOV.radians/2;


		for (int ii=y; ii < y+z; ii++)
		{

			int count =ii-(y);
			double position = half - (da * count);

			double angle = (position < 0) ? position + 2*Math.PI : position;
			locations[ii] = this.computeLocationForHole(globe, Angle.fromRadians(angle));
		}
		//Outside right side circle
		da = this.FOV.radians / 2 / (y-1);
		double start = (2*Math.PI) - this.FOV.radians/2;

		for (int iii=y+z; iii < numPositions; iii++)
		{

			int count = iii-(y+z);
			double angle = (iii != numPositions-1) ? start + count * da : 0;
			locations[iii] = this.computeLocationFor(globe, Angle.fromRadians(angle));

		}

		return java.util.Arrays.asList(locations);
    }


	//Override SensorEllipse method
	@SuppressWarnings("deprecation")
	protected Iterable<LatLon> computeLocations(Globe globe, int intervals)
	    {


	        if (globe == null)
	        {
	            String message = Logging.getMessage("nullValue.GlobeIsNull");
	            Logging.logger().severe(message);
	            throw new IllegalArgumentException(message);
	        }

	        int numPositions = 1 + Math.max(ELLIPSE_MIN_NUM_INTERVALS, intervals);
	        double da = (2 * Math.PI) / (numPositions - 1);
	        LatLon[] locations = new LatLon[numPositions];

	        for (int i = 0; i < numPositions; i++)
	        {
	            double angle = (i != numPositions - 1) ? i * da : 0;
	            locations[i] = this.computeLocationFor(globe, Angle.fromRadians(angle));
	        }



	        return java.util.Arrays.asList(locations);
    }

	//Override SensorEllipse method
    @SuppressWarnings("deprecation")
	public Iterable<? extends LatLon> getLocations(Globe globe)
	{


		if (globe == null)
		{
			String message = Logging.getMessage("nullValue.GlobeIsNull");
			Logging.logger().severe(message);
			throw new IllegalArgumentException(message);
		}

		return this.computeLocationsHole(globe, this.intervals);
    }

	//Override SensorEllipse method
    @SuppressWarnings("deprecation")
	protected Iterable<? extends LatLon> getLocations(Globe globe, double edgeIntervalsPerDegree)
    {

        if (globe == null)
        {
            String message = Logging.getMessage("nullValue.GlobeIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        int intervals = this.computeNumIntervals(globe, edgeIntervalsPerDegree);
        return this.computeLocationsHole(globe, intervals);
    }

	//Override SensorEllipse method
	@SuppressWarnings("deprecation")
	protected LatLon computeLocationFor(Globe globe, Angle angle)
    {

        if (globe == null)
        {
            String message = Logging.getMessage("nullValue.GlobeIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        if (angle == null)
        {
            String message = Logging.getMessage("nullValue.AngleIsNull");
            Logging.logger().severe(message);
            throw new IllegalArgumentException(message);
        }

        double xLength = this.majorRadius * Math.cos(angle.radians);
        double yLength = this.minorRadius * Math.sin(angle.radians);
        double distance = Math.sqrt(xLength * xLength + yLength * yLength);
        double globeRadius = globe.getRadiusAt(this.center.getLatitude(), this.center.getLongitude());

        // azimuth runs positive clockwise from north and through 360 degrees.
        double azimuth = (Math.PI / 2.0) - (Math.acos(xLength / distance) * Math.signum(yLength)
            - this.heading.radians);

        return LatLon.greatCircleEndPosition(this.center, azimuth, distance / globeRadius);
    }

	//Override SensorEllipse method
    /*protected Iterable<? extends Sector> computeBoundingSectors(Globe globe)
       {

           if (globe == null)
           {
               String message = Logging.getMessage("nullValue.GlobeIsNull");
               Logging.logger().severe(message);
               throw new IllegalArgumentException(message);
           }

           // TODO: Compute a better fitting bounding sector for SurfaceEllipse.
           double radius = Math.max(this.majorRadius, this.minorRadius);
           return java.util.Arrays.asList(Sector.splitBoundingSectors(globe, this.center, radius));
    }*/

}

